library(ggplot2)
library(MuMIn)
library (phylolm)
library(recluster)
library(rworldmap)
library(rworldxtra)
newmap <- getMap(resolution = "high")
library(plotrix)
library(pegas)
library(phangorn)
library(phytools)
library(raster)
library(caper)

source("new_functions.R")
sea<-shapefile("ne_10m_ocean.shp")

data<-read.table("data_specimens.txt", h=T, sep="\t")
sequences<-read.FASTA("data_sequences.fas")
length(sequences)
nrow(data)
head(data)

#Obtain spcies list
species<-data[!duplicated(data[,8]),8]

#Divide areas in islands and mainland areas in 2.5 deg units

islandlat<-aggregate(data[,12]~ data[,9], FUN="mean")
islandlong<-aggregate(data[,13]~ data[,9], FUN="mean")
data<-cbind(data,rep(0,nrow(data)))
data<-cbind(data,rep(0,nrow(data)))

for (i in 1:nrow(islandlong)){
	data[which(data[,9]==islandlong[i,1]),17]<-islandlong[i,2]
}
for (i in 1:nrow(islandlat)){
	data[which(data[,9]==islandlat[i,1]),16]<-islandlat[i,2]
}

lat<-floor(data[,12]/2.5)*2.5
long<-floor(data[,13]/2.5)*2.5

# Use the two lines below to obtain the zoogeographic maps with collapsed small squares
#lat<-data[,14]
#long<-data[,15]

#Recognize specimens from mainland and large islands
areascoll<-c("main", "Corsica", "Sardinia","Sicily", "Britain", "Ireland")
mainvect<-NULL
pos<-1
for(t3 in 1:nrow(data)){
	test<-which(grepl(data[t3,9],areascoll))
	if (length(test)>0){
		mainvect[pos]<-t3
	pos<-pos+1
	}
}
main<-data[mainvect,]
mainlat<-aggregate(main[,12]~ lat[mainvect]+long[mainvect]+main[,11], FUN="mean")
mainlat<-cbind(mainlat,aggregate(main[,13]~ lat[mainvect]+long[mainvect]+main[,11], FUN="mean")[,4])

#Attribute average coordinates among specimens to each area
for(i in 1 : nrow(data)){
	for(k in 1 : nrow(mainlat)){
		if (lat[i]==mainlat[k,1]){
			if (long[i]==mainlat[k,2]){
				if (data[i,11]==mainlat[k,3]){
					data[i,16]<-mainlat[k,4]
					data[i,17]<-mainlat[k,5]
				}
			}
		}
	}
}

#attribute a number to each area
sites<-aggregate(data[,14]~lat+long+data[,11],FUN="mean")
sites[,4]<-c(1:nrow(sites))

val<-NULL
for (k in 1:nrow(data)){
	for(ar in 1:nrow(sites)){
		if(lat[k]==sites[ar,1]){
			if(long[k]==sites[ar,2]){
				if(data[k,11]==sites[ar,3]){
					val[k]<-sites[ar,4]
				}
			}
		}
	}
}

data<-cbind(data,val)
write.table(data, "data_specimens_with_areas.txt", sep="\t")
data<-read.table("data_specimens_with_areas.txt", sep="\t", h=T)


# Recognize how many land units and sequences are available for each species

areas<-NULL
seqs<-NULL
for (k in 1:length(species)){
	select<-which(data[,8]==species[k])
	data2<-data[select,]
    	areas[k]<-length(which(!duplicated(data2[,18])))
    	seqs[k]<-length(select)
}

# Select the species
species_use<-species[which(areas>3 & seqs>14)]
length(species_use)
species_use<-species_use[order(species_use)]

# Open the traits
traits<-read.table("traits.txt", h=T, sep="\t")

# Calculate indexes of genetic differentiation
specimens<-NULL
areas<-NULL
nucdiv<-NULL
Gst<-NULL
Dst<-NULL
G1st<-NULL
D<-NULL
piante<-NULL
tratti<-traits[1,]
head(data)
for (cou in 1: length(species_use)){
	select<-which(data[,8]==as.character(species_use[cou]))
	seqs<-sequences[select]
	datasel<-data[select,]
	specimens[cou]<-length(select)
	areas[cou]<-length(!duplicated(datasel[,18]))
	distances <-dist.dna(seqs, model = "raw",pairwise.deletion = TRUE)
	fst<-recluster.fst(distances, datasel[,18],setzero=T)
	Gst[cou]<-fst$Gst
	G1st[cou]<-fst$G1st
	D[cou]<-fst$D
	Dst[cou]<-fst$Dst
	hapl<-recluster.haplotypes(seqs)
	nucdiv[cou]<-nuc.div(hapl$seqs)
	sel3<-which(match(traits[,4],species_use[cou])==1)
	if(length(sel3)!=1){
		tratti<-rbind(tratti, rep(NA,ncol(traits)))
	}
	if(length(sel3)==1){
		tratti<-rbind(tratti, traits[sel3,])
	}
}
tratti<-tratti[-1,]
mean(specimens)
sd(specimens)
hist(specimens,breaks=40)

tab<-cbind(Gst,G1st)
tab<-tab[complete.cases(tab),]
cor.test(tab[,1],tab[,2])
plot(tab[,1],tab[,2])
abline(lm(tab[,2]~(tab[,1])))

tabfinal<-cbind(species_use,Gst,sqrt(Dst),sqrt(nucdiv),sqrt(D),G1st,tratti)
colnames(tabfinal)[1]<-"species"
head(tabfinal)

# Remove species with incomplete data
tabfinal <-tabfinal[complete.cases(tabfinal),]
head(tabfinal)
#Make PCA for phenology data
datafen<-(tabfinal[,c(15,16,17,19)])
pcafen<-rda(datafen,scale=T)
pcafen$CA$eig
biplot(pcafen,scaling=3)
phenology<-pcafen$CA$u[,1]
pcafen$CA$v
plot(cbind(c(1:length(pcafen$CA$eig)),pcafen$CA$eig),cex=1.5, xlab="PC", ylab="Eigenvalue")
lines(cbind(c(1:length(pcafen$CA$eig)),pcafen$CA$eig))
arrows(1,1,4,1,length=0)


#PCA for ecophysiology
dataeco<-(tabfinal[,c(25,27,28,29,30,32)])
head(dataeco)
pcaeco<-rda(dataeco,scale=T)
pcaeco$CA$eig
biplot(pcaeco,scaling=3)
ecophysio<-pcaeco$CA$u[,1:2]
pcaeco$CA$v
plot(cbind(c(1:length(pcaeco$CA$eig)), pcaeco$CA$eig),cex=1.5, xlab="PC", ylab="Eigenvalue")
lines(cbind(c(1:length(pcaeco$CA$eig)), pcaeco$CA$eig))
arrows(1,1,4,1,length=0)


# Merge the PCs
tabfinal<-cbind(tabfinal,phenology,ecophysio)
rownames(tabfinal)<-tabfinal[,1]
colnames(tabfinal)[34:36]<-c("phenology_PC1", "Ecophysiology_PC1","Ecophysiology_PC2")
write.table(tabfinal,"tabfinal.txt")
tabfinal<-read.table("tabfinal.txt")


# Open the tree for the Phylogenetic regression
tree<-read.tree("Butterflies_Europe_tree.nwk")
treeRegr <-force.ultrametric(tree,method="extend")
write.tree(treeRegr,"Butterflies_Europe_tree_ultrametric.nwk")


# Prepare the tree by removing the species not in the analysis
newtree<-drop.tip(treeRegr,treeRegr$tip.label[-match(tabfinal[,1], treeRegr$tip.label)])
plot(newtree, cex=0.3)
head(tabfinal)
table2<-tabfinal[,c(2:6,13,22,24,33:36)]

# Transform and scale the data
table2$plant_genera<-log(table2$plant_genera)
table2$wing_average<-log(table2$wing_average)
table2$range_size<-log(table2$range_size)
table2$phenology_PC1<--table2$phenology_PC1

nam<-newtree$tip.label
table2<-table2[match(nam,rownames(table2)),]
table3<-table2
for (i in 1:ncol(table2)){
	table3[,i]<-scale(table3[,i], scale=T,center=T)
}
rownames(table3)<-rownames(table2)

head(table3)


table3<-cbind(table3,rownames(table3))
colnames(table3)[ncol(table3)]<-"Species"

butterflies <- comparative.data(newtree, table3, Species, vcv=TRUE, vcv.dim=3)


# Change the name of the variable var to test for Phylogenetic signal in all variable
var<-
table3$wing_average
row.names(var)<-rownames(tabfinal)
phylosig(newtree, var , method="lambda", test=T, nsim=1000, se=NULL, start=NULL,
	control=list())

# Perform the phylogenetic regression

pgls<-pgls(Gst~plant_genera+wing_average+phenology_PC1+range_size+Ecophysiology_PC1+Ecophysiology_PC2+altitude_max,butterflies,lambda='ML')
summary(pgls)


options(na.action = "na.fail") 
ms1 <- dredge(pgls) 
subset(ms1, delta < 4) 
summary(model.avg(subset(ms1, delta < 4)))



head(table2)
ggplot(table2, aes(x=plant_genera, y=Gst)) + 
    geom_smooth(method=lm, fill="green",color="darkgreen")
    
ggplot(table2, aes(x=range_size, y=Gst)) + 
    geom_smooth(method=lm, fill="red",color="darkred")
    
ggplot(table2, aes(x=wing_average, y=Gst)) + 
    geom_smooth(method=lm, fill="blue",color="darkblue")

ggplot(table2, aes(x=phenology_PC1, y=sqrt.Dst.)) + 
    geom_smooth(method=lm, fill="orange",color="darkorange")


# Make the Zoogeographic assessments

areasstart<-max(data[,ncol(data)])

# Create empty matrices for inter-area Gst for each species
matrixGst<-array(NA,dim=c(areasstart,areasstart,length(species_use)))
areatabledissGst<-array(NA,dim=c(areasstart,areasstart))
for (sp in 1:length(species_use)){
	sel<-which(data[,8]==species_use[sp])
	fastared<-sequences[sel]
	datamatsp<-data[sel,ncol(data)]
	dismatsp<-dist.dna(fastared, model = "raw",pairwise.deletion = TRUE)
	areamatsp<-datamatsp[!duplicated(datamatsp)]
	dista<-recluster.fst.pair(as.dist(dismatsp),datamatsp,setzero=T)

#Then, the dissimilarity values are arranged with the same order in the MatrixGst object

	for (n in 1: length(areamatsp)){
		for (m in 1:length(areamatsp)){
			matrixGst[areamatsp[n], areamatsp[m],sp]<-as.matrix(dista$Gstm)[n,m]
			if(as.matrix(dista$Gstm)[n,m]=="NaN"){
				matrixGst[areamatsp[n], areamatsp[m],sp]<-NA
			}
		}
	}
}

count<-areatabledissGst
countzero<-array(0,dim=c(areasstart,areasstart))

# Now the mean values among the matrices of the "matrix" object are computed and stored in areatablediss
for (r in 1: areasstart){
	for (c in 1: areasstart){
		count[r,c]<-length(which(!is.na(matrixGst[r,c,])))
		if(count[r,c]>1){
			areatabledissGst[r,c]<-mean(matrixGst[r,c,],na.rm=T)
			countzero[r,c]<-1
		}
	}
}

rownames(areatabledissGst)<-c(1:areasstart)
colnames(areatabledissGst)<-c(1:areasstart)
rownames(matrixGst)<-c(1:areasstart)
colnames(matrixGst)<-c(1:areasstart)

sum<-colSums(count)
sumzero<-areasstart-colSums(countzero)

areatabledissGst2<-areatabledissGst
matrixGst2<-matrixGst

for (i in 1:nrow(areatabledissGst2)){
    if(sum(sumzero)>0){
    remove<-which(sumzero==max(sumzero))
	areatabledissGst2<-areatabledissGst2[-remove,-remove]
	matrixGst2<-matrixGst2[-remove,-remove,]
	count3<-matrix(NA,ncol(areatabledissGst2),ncol(areatabledissGst2))
	countzero3<-matrix(0,ncol(areatabledissGst2),ncol(areatabledissGst2))
		for (r in 1: ncol(areatabledissGst2)){
			for (c in 1: ncol(areatabledissGst2)){
				count3[r,c]<-length(which(!is.na(matrixGst2[r,c,])))
				if(count3[r,c]>9){
					areatabledissGst2[r,c]<-mean(matrixGst2[r,c,],na.rm=T)
					countzero3[r,c]<-1
				}
			}
		}
	sumzero<-ncol(areatabledissGst2)-colSums(countzero3)
	}
}

# Calculate the MDS configuration to obtain the colours for individual areas
pcoaGSt<-metaMDS(as.dist(sqrt(areatabledissGst2)))$points
colfinalGst<-recluster.col(pcoaGSt[,c(2,1)],st=F,rot=F)
recluster.plot.col(colfinalGst,text=F,cex=1.5)

# Prepare the matrix of the areas and separate islands on the grid
xy<-matrix(NA, nrow(areatabledissGst2),5)
for (i in 1:nrow(areatabledissGst2)){
	whicharea<-which(data[,ncol(data)]==rownames(areatabledissGst2)[i])
	sel<-cbind(lat[whicharea],long[whicharea],data[whicharea,c(16:17,9)])
	xy[i,1]<-(sel[1,1])
	xy[i,2]<-(sel[1,2])
	xy[i,3]<-(sel[1,3])
	xy[i,4]<-(sel[1,4])
	xy[i,5]<-as.character(sel[1,5])
}
squares<-which (xy[,5]%in% c("main", "Corsica", "Sardinia", "Sicily"))
squarescoord<-xy[squares,]
squarescol<-colfinalGst[squares,]
islandscoord<-xy[-squares,]
islandscol<-colfinalGst[-squares,]
corsicaS<-which(squarescoord[,1]==40 & squarescoord[,2]==7.5 & squarescoord[,5]=="Corsica")
aspromonte<-which(squarescoord[,5]=="main" & squarescoord[,1]==37.5 & squarescoord[,2]==15)


# Calculate the tree
tree<-as.phylo(hclust(as.dist(areatabledissGst2),method="ward.D"))
plot(tree)

# Cut the tree 
gru<-recluster.expl.diss(tree,as.dist((areatabledissGst2)))

# Select the number of clusters
k<-2

col2<-recluster.group.col(colfinalGst,gru$matrix[,k-1])$all
squarescol<-col2[squares,]
islandscol<-col2[-squares,]


plot(rbind(c(-10, 35),c(20, 50)),type="n")
for (sq in 1:nrow(squarescoord)){
	rect(squarescoord[sq,2],squarescoord[sq,1],as.numeric(squarescoord[sq,2])+2.5,as.numeric(squarescoord[sq,1])+2.5,  col = rgb(squarescol[sq, 3], squarescol[sq, 4], squarescol[sq, 5], maxColorValue = 255))
}
rect(7.5,41.3,10,42.5,col = rgb(squarescol[corsicaS, 3], squarescol[corsicaS, 4], 
            squarescol[corsicaS, 5], maxColorValue = 255))
rect(15.60,37.5,17.5,40,col = rgb(squarescol[aspromonte, 3], squarescol[aspromonte, 4], squarescol[aspromonte, 5], maxColorValue = 255))

plot(newmap, xlim = c(-10, 20), ylim = c(35, 42), asp = 1, add=T)
plot(sea,xlim = c(-10, 20), ylim = c(35, 42),col=rgb(192, 218, 242, maxColorValue = 255),add=T)
points(as.numeric(islandscoord[,4]), as.numeric(islandscoord[,3]),cex=2.1)
recluster.plot.sites.col(as.numeric(islandscoord[,4]), as.numeric(islandscoord[,3]), islandscol,add=T,cext=2,pch=16,cex=2)





