recluster.plot.pie<-function(long, lat, loc, col, map=NULL,add=F,size=T,minsize=NULL,xylim=NULL,main=NULL,xlab=NULL,ylab=NULL,mgp=mgp){
	if(is.null(xylim)){
		xylim<-c(range(long),range(lat))
		}
	if(!(add)){
		plot(cbind(xylim[1:2],xylim[3:4]),type="n",main=main,xlab=xlab,ylab=ylab)
	}
	if(!is.null(map)){
		plot(map, asp = 2,add=T,xlab=xlab,ylab=ylab)
	}
	if(is.null(minsize)){
		minsize<-min(abs(range(long)[1]-range(long)[2]),abs(range(lat)[1]-range(lat)[2]))/30
	}
	for (i in 1:max(loc)){
		specim<-which(loc==i)
		if(length(specim)==1){
			specimens<-c(long[specim], lat[specim],col[specim,])
			floating.pie(specimens[1],specimens[2],1,radius=minsize,border=NA,col=rgb(specimens[5], specimens[6], specimens[7], maxColorValue = 255))
			draw.circle(specimens[1],specimens[2],radius=minsize)
		}
	if(length(specim)>1){
		specimens<-cbind(long[specim], lat[specim],col[specim,])
		if(length(specim)>3){
			dista<-dist(col[specim,1:2])
			if(sum(dista)>0){
				mds<-cbind(c(1:length(specim)),cmdscale(dist(col[specim,1:2]),k=1))
				specimens<-specimens [order(mds[,2]),]
				}
			}
		floating.pie(mean(specimens[,1]),mean(specimens[,2]),rep(1,length(specim)),border=NA,radius=minsize*(length(specim))^0.25,col=rgb(specimens[, 5], specimens[, 6], specimens[, 7], maxColorValue = 255))
		draw.circle(mean(specimens[,1]),mean(specimens[,2]),radius=minsize*(length(specim))^0.25)
		}
	}
}

recluster.fst<-function(dist,vect,pairwise=F,setzero=F){
	res<-NULL
	populations<-vect[!duplicated(vect)]
	spec<-length(populations)
	vectmat<-as.vector(dist)
	Ht<-mean(dist)
	res$Ht<-Ht
	res$lengthHt<-length(dist)
	omol<-as.vector(dist(vect))
	omol[which(omol>0)]<-1
	res$lengthHs<-length(vectmat[which(omol==0)])
	Hs<-mean(vectmat[which(omol==0)],na.rm=T)
	res$Hs<-Hs
	res$Gst<-((Ht-Hs)/(Ht))
	res$G1st<-((Ht-Hs)/(Ht))/(((spec-1)*(1-Hs))/(spec-1+Hs))
	res$Dst<-Ht-Hs
	res$D<-((Ht-Hs)/(1-Hs))*(spec/(spec-1))
	if (setzero){
		res$Gst[res$Gst<0]<-0
		res$Dst[res$Dst<0]<-0
		res$D[res$D<0]<-0
		res$G1st[res$G1st<0]<-0
		}
	return(res)
}


recluster.fst.pair<-function(dist,vect,setzero=F){
	res<-NULL
	populations<-vect[!duplicated(vect)]
	matrix<-as.matrix(dist)
	matrix2<-matrix(NA, length(populations),length(populations))
	colnames(matrix2)<-populations
	rownames(matrix2)<-populations
	matrix3<-matrix2
	phist<-recluster.fst(dist,vect,setzero=setzero)		
	for(col in 1:(length(populations)-1)){
			for(row in (col+1) :length(populations)){
				sum<-NULL
				vectsub<-vect[which(vect==populations[row]|vect==populations[col])]
				subm<-matrix[which(vect==populations[row]|vect==populations[col]),which(vect==populations[row]|vect==populations[col])]
				submfst<-recluster.fst(as.dist(subm), vectsub)
				matrix2[row,col]<-submfst$Dst/phist$Ht
				matrix3[row,col]<-submfst$Dst
				}
			}	
			
	res$Ht<-phist$Ht
	res$Hs<-phist$Hs
	res$Gst<-phist$Gst
	res$Dst<-phist$Dst
	res$Gstm<-as.dist(matrix2)
	res$Dstm<-as.dist(matrix3)
	if (setzero){
		res$Gst[res$Gst<0]<-0
		res$Dst[res$Dst<0]<-0
		res$Gstm[res$Gstm<0]<-0
		res$Dstm[res$Dstm<0]<-0
	}
return (res)
}

triang.test<-function(mat,nullmat=1000,sel=NULL,cex=1,pch1=19,pch2=1,method="variance",text=T,absent=T){
if (!is.vector(sel)){sel<-c(1:nrow(mat))}
res<-NULL
mata<-mat[sel,]
varx<-max(mat[,1])-min(mat[,1])
vary<-max(mat[,2])-min(mat[,2])

if(method=="mean"){
limy<-mean(mat[,2])
limx<-mean(mat[,1])
}
if(method=="variance"){
limy<-min(mat[,2])+vary/2
limx<-min(mat[,1])+varx/2
}
if(method=="median"){
limy<-median(mat[,2])
limx<-median(mat[,1])
}
q<-mata[mata[,1]>=limx,2]
q1<-length(q[q>limy])
q<-mata[mata[,1]<limx,2]
q2<-length(q[q>limy])
q<-mata[mata[,1]<limx,2]
q3<-length(q[q<=limy])
q<-mata[mata[,1]>=limx,]
q4<-length(q[q<=limy])
v<-c(q1,q2,q3,q4)
matr<-matrix(NA,4,nullmat+1)
matr[,1]<-v
res$obs<-v
for (rep in 1:nullmat){
if(method=="mean"){
y<-rnorm(nrow(mata), mean=mean(mat[,2]),sd=sd(mat[,2]))
x<-rnorm(nrow(mata), mean=mean(mat[,1]),sd=sd(mat[,1]))
}
if(method=="median"){
y<-rnorm(nrow(mata), mean=mean(mat[,2]),sd=sd(mat[,2]))
x<-rnorm(nrow(mata), mean=mean(mat[,1]),sd=sd(mat[,1]))
}
if(method=="variance"){
y<-runif(nrow(mata), min(mat[,2]),max(mat[,2]))
x<-runif(nrow(mata), min(mat[,1]),max(mat[,1]))
}
mat2<-cbind(x,y)
q<-mat2[mat2[,1]>=limx,2]
q1<-length(q[q>limy])
q<-mat2[mat2[,1]<limx,2]
q2<-length(q[q>limy])
q<-mat2[mat2[,1]<limx,2]
q3<-length(q[q<=limy])
q<-mat2[mat2[,1]>=limx,2]
q4<-length(q[q<=limy])
matr[,rep+1]<-c(q1,q2,q3,q4)
}
res$mean<-rowMeans(matr[,2:ncol(matr)])
res$pval<-NULL
for (i in 1:4){
comp<-matr[i,2:ncol(matr)]
res$pval[i]<-length(comp[comp<matr[i,1]])/nullmat
}
val<-c(2,3,2,1)
val2<-c(1,2,1,0)
res$index<-sum(matr[,1]*val)/sum(sum(matr[,1])*max(val))
res$index2<-sum(matr[,1]*val2)/sum(sum(matr[,1])*max(val2))
plot(mat,type="n",xlab="",ylab="")
points (mata,pch=pch1,cex=cex)
if (absent)
{points (mat[-sel,],pch=pch2,cex=cex)}
arrows(limx-0.2,min(mat[,2])-2,limx-0.2,max(mat[,2])+2,length=0)
arrows(min(mat[,1])-2,limy,max(mat[,1])+2,limy,length=0)
if (text){
text (1.5*limx,1.5*limy,paste(matr[1,1],"->",res$pval[1]),cex=0.7)
text (0.5*limx,1.5*limy,paste(matr[2,1],"->",res$pval[2]),cex=0.7)
text (0.5*limx,0.5*limy,paste(matr[3,1],"->",res$pval[3]),cex=0.7)
text (1.5*limx,0.5*limy,paste(matr[4,1],"->",res$pval[4]),cex=0.7)
}
return(res)
}


recluster.haplotypes<-function(fasta,pop=NULL){
number<-c(1:length(fasta))
len<-NULL
for (n in 1:length(fasta)){
	leng1<-length(which(fasta[[n]]=="-"))
	leng2<-length(which(fasta[[n]]=="N"))
	leng3<-length(which(fasta[[n]]=="n"))
	leng4<-length(which(fasta[[n]]=="f0"))
	len[n]<-length(fasta[[n]])-(leng1+leng2+leng3+leng4)
	}
matrdataor<-number[order(len,decreasing = T)]
fastaor<-fasta[order(len,decreasing = T)]
res<-NULL
matrdata<-matrix(0,length(fasta),6)
rownames(matrdata)<-names(fastaor)
matrdata[,1]<-matrdataor
sequ<-fastaor[1]
vector<-NULL
vectnames<-NULL
vecthaplotypes<-NULL
vectnames<-names(fastaor[1])
vecthaplotypes[1]<-1
pos<-1
for (n in 2:length(matrdataor)){
	check<-0
	for (giro in 1:(n-1)){
	fastacheck<-c(fastaor[giro],fastaor[n])
	distn<-dist.dna(fastacheck, model = "raw",pairwise.deletion = TRUE)
	if(distn==0){
	vecthaplotypes[n]<-vecthaplotypes[giro]
	vectnames[n]<-vectnames[giro]
	check<-1
	break
	}
	}
	if(check==0){
	pos<-pos+1
	sequ<-c(sequ, fastaor[n])
	vecthaplotypes[n]<-pos
	vectnames[n]<-names(fastaor[n])
	#print(n)
	}
}
res$seqs<-sequ
res$data_final<-cbind(matrdata[,1],vecthaplotypes,vectnames)
res$frequency<-aggregate(rep(1,length(vecthaplotypes))~vecthaplotypes,FUN="sum")[,2]
if(!is.null(pop)){
	popu<-unique(pop)
	haplo<-unique(vecthaplotypes)
	matrpop<-matrix(0,length(popu),length(haplo))
	for(spec in 1:length(vecthaplotypes)){
		#spec<-1
		matrpop[which(popu==pop[spec]),which(haplo==vecthaplotypes[spec])]<-matrpop[which(popu==pop[spec]),which(haplo==vecthaplotypes[spec])]+1
		rownames(matrpop)<-popu
		colnames(matrpop)<-haplo
	}
res$populations<-matrpop
}
return(res)
}


read.alignment <- function(file, format, forceToLower = TRUE)

{

  #

  # Check that we have read permission on the file:

  # 

 



  

  fasta2ali <- function(file){

  	tmp <- file

  	list(length(tmp), getName(tmp), unlist(getSequence(tmp, as.string = TRUE)))

  }

  ali <- switch( tolower(format),

	fasta = fasta2ali(file), 

	mase = .Call("read_mase", file, PACKAGE = "seqinr"),

	phylip = .Call("read_phylip_align", file, PACKAGE = "seqinr"),

	msf = .Call("read_msf_align", file, PACKAGE = "seqinr"),

	clustal = .Call("read_clustal_align", file, PACKAGE = "seqinr"),

	stop("Wrong format name: Format available are fasta, mase, phylip, msf, clustal")

  )



  ali <- lapply(ali, as.character)

  #cleaning \r char

  ali <- lapply(ali, function (x ){gsub ('\r','',x)})

  if(forceToLower) ali[[3]] <- lapply(ali[[3]], tolower)

  if(format == "mase"){

    ali <- list(nb = as.numeric(ali[[1]]), nam = ali[[2]], seq = ali[[3]], com = ali[[4]]) 

  } else {

    ali <- list(nb = as.numeric(ali[[1]]), nam = ali[[2]], seq = ali[[3]], com = NA)

  }

  class(ali) <- "alignment"

  return(ali)

}
